@@ -21,10 +21,11 @@
 */
 
 #include "udisksdevicebackend.h"
+#include "udisks_debug.h"
 
-#include <QtDBus/QDBusConnection>
-#include <QtDBus/QDBusInterface>
-#include <QtXml/QDomDocument>
+#include <QDBusConnection>
+#include <QDBusInterface>
+#include <QDomDocument>
 
 #include "solid/deviceinterface.h"
 #include "solid/genericinterface.h"
@@ -36,14 +37,13 @@ QMap<QString /* UDI */, DeviceBackend *> DeviceBackend::s_backends;
 
 DeviceBackend *DeviceBackend::backendForUDI(const QString &udi, bool create)
 {
-    DeviceBackend *backend = 0;
+    DeviceBackend *backend = nullptr;
     if (udi.isEmpty()) {
         return backend;
     }
 
-    if (s_backends.contains(udi)) {
-        backend = s_backends.value(udi);
-    } else if (create) {
+    backend = s_backends.value(udi);
+    if (!backend && create) {
         backend = new DeviceBackend(udi);
         s_backends.insert(udi, backend);
     }
@@ -53,11 +53,7 @@ DeviceBackend *DeviceBackend::backendForUDI(const QString &udi, bool create)
 
 void DeviceBackend::destroyBackend(const QString &udi)
 {
-    if (s_backends.contains(udi)) {
-        DeviceBackend *backend = s_backends.value(udi);
-        s_backends.remove(udi);
-        delete backend;
-    }
+    delete s_backends.take(udi);
 }
 
 DeviceBackend::DeviceBackend(const QString &udi)
@@ -91,7 +87,7 @@ void DeviceBackend::initInterfaces()
 
     const QString xmlData = introspect();
     if (xmlData.isEmpty()) {
-        qDebug() << m_udi << "has no interfaces!";
+        qCDebug(UDISKS2) << m_udi << "has no interfaces!";
         return;
     }
 
@@ -144,11 +140,15 @@ QVariantMap DeviceBackend::allProperties() const
         QDBusPendingReply<QVariantMap> reply = QDBusConnection::systemBus().call(call);
 
         if (reply.isValid()) {
-            m_propertyCache.unite(reply.value());
+            auto props = reply.value();
+            // Can not use QMap<>::unite(), as it allows multiple values per key
+            for (auto it = props.cbegin(); it != props.cend(); ++it) {
+                m_propertyCache.insert(it.key(), it.value());
+            }
         } else {
-            qWarning() << "Error getting props:" << reply.error().name() << reply.error().message();
+            qCWarning(UDISKS2) << "Error getting props:" << reply.error().name() << reply.error().message();
         }
-        //qDebug() << "After iface" << iface << ", cache now contains" << m_cache.size() << "items";
+        //qDebug() << "After iface" << iface << ", cache now contains" << m_propertyCache.size() << "items";
     }
 
     return m_propertyCache;
@@ -182,15 +182,19 @@ void DeviceBackend::checkCache(const QString &key) const
         return;
     }
 
-    QVariant reply = m_device->property(key.toUtf8());
-    m_propertyCache.insert(key, reply);
+    QDBusMessage call = QDBusMessage::createMethodCall(UD2_DBUS_SERVICE, m_udi, DBUS_INTERFACE_PROPS, "Get");
+    /*
+     * Interface is set to an empty string as in this QDBusInterface is a meta-object of multiple interfaces on the same path
+     * The DBus properties also interface supports this, and will find the appropriate interface if none is explicitly set.
+     * This matches what QDBusAbstractInterface would do
+     */
+    call.setArguments(QVariantList() << QString() << key);
+    QDBusPendingReply<QVariant> reply = QDBusConnection::systemBus().call(call);
 
-    if (!reply.isValid()) {
-        /* Store the item in the cache anyway so next time we don't have to
-         * do the DBus call to find out it does not exist but just check whether
-         * prop(key).isValid() */
-//         qDebug() << m_udi << ": property" << key << "does not exist";
-    }
+    /* We don't check for error here and store the item in the cache anyway so next time we don't have to
+     * do the DBus call to find out it does not exist but just check whether
+     * prop(key).isValid() */
+    m_propertyCache.insert(key, reply.value());
 }
 
 void DeviceBackend::slotPropertiesChanged(const QString &ifaceName, const QVariantMap &changedProps, const QStringList &invalidatedProps)
@@ -244,4 +248,10 @@ void DeviceBackend::slotInterfacesRemoved(const QDBusObjectPath &object_path, co
     Q_FOREACH (const QString &iface, interfaces) {
         m_interfaces.removeAll(iface);
     }
+
+    // We don't know which property belongs to which interface, so remove all
+    m_propertyCache.clear();
+    if (!m_interfaces.isEmpty()) {
+        allProperties();
+    }
 }
