@@ -33,7 +33,6 @@ class RequestStateJob;
 class SOLID_EXPORT Power : public QObject
 {
     Q_OBJECT
-    Q_FLAGS(States)
 public:
     /**
      * List of states a device can be in
@@ -57,6 +56,7 @@ public:
         Shutdown = 1 << 3
     };
     Q_DECLARE_FLAGS(InhibitionTypes, InhibitionType)
+    Q_FLAG(InhibitionTypes)
     /**
      * Returns an instance of Power
      *
@@ -77,26 +77,26 @@ public:
      * The returned AcPluggedJob has to be started, when finished
      * the Job::result() signal will be emitted.
      */
-    static AcPluggedJob* isAcPlugged(QObject *parent = 0);
+    static AcPluggedJob* isAcPlugged(QObject *parent = nullptr);
 
     /**
      * Returns an InhibitionJob
      *
      * The returned job is initialized with the given @p states and @p description
      */
-    static InhibitionJob* inhibit(Power::InhibitionTypes states, const QString &description, QObject *parent = 0);
+    static InhibitionJob* inhibit(Power::InhibitionTypes states, const QString &description, QObject *parent = nullptr);
 
     /**
      * Query the supported states (like Sleep or Hibernation)
      * @return a StatesJob
      */
-    static StatesJob* supportedStates(QObject *parent = 0);
+    static StatesJob* supportedStates(QObject *parent = nullptr);
 
     /**
      * Set the computer in a desired @p state (like Sleep or Hibernation)
      * @return a RequestStateJob
      */
-    static RequestStateJob* requestState(Power::InhibitionType state, QObject *parent = 0);
+    static RequestStateJob* requestState(Power::InhibitionType state, QObject *parent = nullptr);
 
     /**
      * If you are not going to destroy this object for the entire
@@ -106,7 +106,7 @@ public:
      * application is only interested in power management during a small
      * period of time and you want to free the memory after using it.
      */
-    explicit Power(QObject* parent = 0);
+    explicit Power(QObject* parent = nullptr);
 
 Q_SIGNALS:
     /**
@@ -115,6 +115,13 @@ Q_SIGNALS:
      */
     void acPluggedChanged(bool plugged);
 
+    /**
+     * Emitted when the system is about to suspend/hibernate
+     *
+     * @since 5.6
+     */
+    void aboutToSuspend();
+
     /**
      * Emitted when the system has just resumed from being suspended/hibernated
      */
